#!/usr/bin/python

import getopt
import sys
import os
import simplejson as json
import time
import signal
#import urllib.request
from urllib.request import Request, urlopen
from pynovatec.helpers import read_json_from_file

#73782e9b2bba5afe48110a97a9cc2204ab46186c2b52c438a2260622f9cec155
#    'host' : "api.machinesense.com",  # Loopback
    
#'ble_path': "/tmp/sensor_status",

config = {
    'host' : "apiv2.machinesense.com",	# NiFi
    'route': "/api/stats",
    'protocol': "https",
    'stats_url':"https://api.machinesense.com/api/stats",
    'frequency': 60,
    'ble_path': "/tmp/sensor_status",
    'token': "73782e9b2bba5afe48110a97a9cc2204ab46186c2b52c438a2260622f9cec155",
    'ble_sensor_command': "ble sensors",
    'simulate': True
}

def load_config(config_file=None):
    global config

    if config_file:
        tmp_config = read_json_from_file(config_file)
        config = {**config,**tmp_config}
    config['stats_url'] = f"{config['protocol']}://{config['host']}{config['route']}"
    print("stats_url: ",config['stats_url'])

def file_fetch_sensors(ble_path="/tmp/sensor_status"):
    d={}
    for root,dirs,files in os.walk(ble_path):
        if dirs:
            continue
        for name in files:
            sensor_mac=os.path.basename(root)
            path=os.path.join(root,name)
            if sensor_mac not in d:
                d[sensor_mac]={}
            base=name.split('.')[0]
            with open(path,"r") as f:
                d[sensor_mac][base]=json.load(f)
    return d

import ubus

def ubus_fetch_sensors(process="ble", command="sensors"):
    ubus.connect("/var/run/ubus/ubus.sock")
    try:
        ubus_result = ubus.call(process,command,{})
        result = ubus_result[0]
    except Exception as e:
        print("Ubus Exception occured: ",str(e))
        result = {}
    ubus.disconnect()
    return result

def get_sensor_status(ble_path="/tmp/sensor_status", ble_command = None):
    result = {}
    if ble_command:    
        x = ble_command.split(" ")
        if len(x) == 2:
            result = ubus_fetch_sensors(*x)
    if not result:
        print("Result is empty..")
        if os.path.isdir(ble_path):
            return file_fetch_sensors(ble_path)
    return result

#Kluge until we have a file that works for this.
def decorate_sensors(sensors):
    now = int(time.time() * 1000)
    for id, item in sensors.items():
        item['status']['sensors'] = []
        if item['status']['type'] == 'vac':
            item['status']['sensors'].append({"type":"vac","last_seen": now})
        elif item['status']['type'] == 'plc':
            item['status']['sensors'].append({"type":"plc","last_seen": now})
        elif item['status']['type'] == 'st':
            item['status']['sensors'].append({"type":"ac","last_seen": now})
            item['status']['sensors'].append({"type":"hy","last_seen": now})
            item['status']['sensors'].append({"type":"at","last_seen": now})
            item['status']['sensors'].append({"type":"gy","last_seen": now})
            item['status']['sensors'].append({"type":"bp","last_seen": now})
            item['status']['sensors'].append({"type":"mg","last_seen": now})
            item['status']['sensors'].append({"type":"ir","last_seen": now})

#    info = {
#        "name": "a11c30b9d288_AC_1_AC",
#        "display_name": "a11c30b9d288_AC_1_AC",
#        "rate": 28,
#        "time": 1656088584,
#        "id": "a11c30b9d288_AC_1_AC"
#    }

def send_sensor_info(sid,last_seen):
    info = {
        "name": sid, 
        "display_name": sid,
        "rate": 28,
        "time": last_seen,
        "id": sid 
    }
    route=f"{config['stats_url']}/{sid}"
    token=config['token']
    headers = {
	'content-type': 'application/json',
        'Token': token
    }
    data = bytes(json.dumps(info),encoding='utf8')
    request = Request(route,method="PUT",data=data,headers=headers)

    res = urlopen(request)
    #with urlopen(request) as response:
    #    print(response.read())

    #req = urllib.request(route)
    #req.add_header('Content-Type','application.json')
    #response = urllib.urlopen(req,json.dumps(info))


def send_sensor_status(sensors):
    for id, item in sensors.items():
        for sensor in item['status']['sensors']:
            idx=sensor.get('index',1)
            ap = f"{sensor['type']}_{idx}_{sensor['type']}".upper()
            tid = id.replace(':','').lower()
            sid = f"{tid}_{ap}"
            # Last seen may be Seconds or Miliseconds so lets divide by current time and see if we are over 100 to determine we are in ms.
            now = int(time.time())
            ls = sensor['last_seen']
            if ls/now > 100:
                ls = sensor['last_seen']/1000
            send_sensor_info(sid,ls)

def read_sensor_info(simulate=False, ble_path="/tmp/sensor_status", ble_command = None):
    sensors = get_sensor_status(ble_path=ble_path, ble_command = ble_command)
    print("Sensors: ", json.dumps(sensors,indent=4))
    #Temporary
    if simulate:
        decorate_sensors(sensors);
    return sensors

from threading import Event
exit = Event()
keep_running = True

def signal_to_quit(signum, frame):
    global keep_running

    exit.set()
    keep_running = False

def signal_to_do_nothing(signum, frame):
    exit.set()
    pass

def signal_init():
    signal.signal(signal.SIGINT,signal_to_quit)
    signal.signal(signal.SIGTERM,signal_to_quit)
    signal.signal(signal.SIGQUIT,signal_to_quit)
    signal.signal(signal.SIGHUP,signal_to_do_nothing)   # Make it so SIGHUP breaks the wait.



def main_loop():
    global config

    while keep_running:
        print("Send Sensor...")
        try:
            sensors = read_sensor_info(ble_path=config['ble_path'],simulate=config['simulate'], ble_command=config['ble_sensor_command'])
            #print("Sensors: ",json.dumps(sensors,indent=4))
            send_sensor_status(sensors)
        except Exception as e:
            print("Exception during sensor process: ",str(e))
        try:
            exit.wait(int(config['frequency']))
        except:
            print("Wait exception..")
        exit.clear()



def main():
    config_file = None
    try:
        opts, args = getopt.getopt(sys.argv[1:],"c:")
    except getopt.GetoptError:
        print("Error: Invalid argument ")
        usage(__file__)
    for opt,arg in opts:
        #print("opt is: ",opt)
        if opt == "-c":
            config_file = arg

    load_config(config_file=config_file)
    signal_init()
    main_loop()

main()


